@@ -148,144 +148,6 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
 
 	// --------------------------JDBC 2.0-----------------------------------
 
-	/**
-	 * JDBC 2.0
-	 * 
-	 * <p>
-	 * Return the fully qualified name of the Java class whose instances are
-	 * manufactured if ResultSet.getObject() is called to retrieve a value from
-	 * the column. ResultSet.getObject() may return a subClass of the class
-	 * returned by this method.
-	 * </p>
-	 * 
-	 * @param column
-	 *            the column number to retrieve information for
-	 * 
-	 * @return the fully qualified name of the Java class whose instances are
-	 *         manufactured if ResultSet.getObject() is called to retrieve a
-	 *         value from the column.
-	 * 
-	 * @throws SQLException
-	 *             if an error occurs
-	 */
-	public String getColumnClassName(int column) throws SQLException {
-		Field f = getField(column);
-
-		// From JDBC-3.0 spec, Table B-3
-		//
-		// JDBC Type Java Object Type
-		//
-		// CHAR String
-		// VARCHAR String
-		// LONGVARCHAR String
-		// NUMERIC java.math.BigDecimal
-		// DECIMAL java.math.BigDecimal
-		// BIT Boolean
-		// BOOLEAN Boolean
-		// TINYINT Integer
-		// SMALLINT Integer
-		// INTEGER Integer
-		// BIGINT Long
-		// REAL Float
-		// FLOAT Double
-		// DOUBLE Double
-		// BINARY byte[]
-		// VARBINARY byte[]
-		// LONGVARBINARY byte[]
-		// DATE java.sql.Date
-		// TIME java.sql.Time
-		// TIMESTAMP java.sql.Timestamp
-		// DISTINCT Object type of underlying type
-		// CLOB Clob
-		// BLOB Blob
-		// ARRAY Array
-		// STRUCT Struct or SQLData
-		// REF Ref
-		// DATALINK java.net.URL
-		// JAVA_OBJECT underlying Java class
-		switch (f.getSQLType()) {
-		case Types.BIT:
-		case Types.BOOLEAN:
-			return "java.lang.Boolean"; //$NON-NLS-1$
-
-		case Types.TINYINT:
-
-			if (!f.isUnsigned()) {
-				return "java.lang.Integer"; //$NON-NLS-1$
-			}
-
-			return "java.lang.Integer"; //$NON-NLS-1$
-
-		case Types.SMALLINT:
-
-			if (!f.isUnsigned()) {
-				return "java.lang.Integer"; //$NON-NLS-1$
-			}
-
-			return "java.lang.Integer"; //$NON-NLS-1$
-
-		case Types.INTEGER:
-
-			if (!f.isUnsigned()) {
-				return "java.lang.Integer"; //$NON-NLS-1$
-			}
-
-			return "java.lang.Long"; //$NON-NLS-1$
-
-		case Types.BIGINT:
-
-			if (!f.isUnsigned()) {
-				return "java.lang.Long"; //$NON-NLS-1$
-			}
-
-			return "java.math.BigInteger"; //$NON-NLS-1$
-
-		case Types.DECIMAL:
-		case Types.NUMERIC:
-			return "java.math.BigDecimal"; //$NON-NLS-1$
-
-		case Types.REAL:
-			return "java.lang.Float"; //$NON-NLS-1$
-
-		case Types.FLOAT:
-		case Types.DOUBLE:
-			return "java.lang.Double"; //$NON-NLS-1$
-
-		case Types.CHAR:
-		case Types.VARCHAR:
-		case Types.LONGVARCHAR:
-			if (!f.isOpaqueBinary()) {
-				return "java.lang.String"; //$NON-NLS-1$
-			}
-
-			return "[B";
-
-		case Types.BINARY:
-		case Types.VARBINARY:
-		case Types.LONGVARBINARY:
-
-			if (f.getMysqlType() == MysqlDefs.FIELD_TYPE_GEOMETRY) {
-				return "[B";
-			} else if (f.isBinary() || f.isBlob()) {
-				return "[B";
-			} else {
-				return "java.lang.String";
-			}
-
-		case Types.DATE:
-			return "java.sql.Date"; //$NON-NLS-1$
-
-		case Types.TIME:
-			return "java.sql.Time"; //$NON-NLS-1$
-
-		case Types.TIMESTAMP:
-			return "java.sql.Timestamp"; //$NON-NLS-1$
-
-		default:
-			return "java.lang.Object"; //$NON-NLS-1$
-		}
-	}
-
 	/**
 	 * Whats the number of columns in the ResultSet?
 	 * 
@@ -800,4 +662,124 @@ public class ResultSetMetaData implements java.sql.ResultSetMetaData {
 
 		return toStringBuf.toString();
 	}
+
+	static String getClassNameForJavaType(int javaType, 
+			boolean isUnsigned, int mysqlTypeIfKnown, 
+			boolean isBinaryOrBlob,
+			boolean isOpaqueBinary) {
+		switch (javaType) {
+		case Types.BIT:
+		case Types.BOOLEAN:
+			return "java.lang.Boolean"; //$NON-NLS-1$
+	
+		case Types.TINYINT:
+	
+			if (isUnsigned) {
+				return "java.lang.Integer"; //$NON-NLS-1$
+			}
+	
+			return "java.lang.Integer"; //$NON-NLS-1$
+	
+		case Types.SMALLINT:
+	
+			if (isUnsigned) {
+				return "java.lang.Integer"; //$NON-NLS-1$
+			}
+	
+			return "java.lang.Integer"; //$NON-NLS-1$
+	
+		case Types.INTEGER:
+	
+			if (!isUnsigned || 
+					mysqlTypeIfKnown == MysqlDefs.FIELD_TYPE_INT24) {
+				return "java.lang.Integer"; //$NON-NLS-1$
+			}
+	
+			return "java.lang.Long"; //$NON-NLS-1$
+	
+		case Types.BIGINT:
+	
+			if (!isUnsigned) {
+				return "java.lang.Long"; //$NON-NLS-1$
+			}
+	
+			return "java.math.BigInteger"; //$NON-NLS-1$
+	
+		case Types.DECIMAL:
+		case Types.NUMERIC:
+			return "java.math.BigDecimal"; //$NON-NLS-1$
+	
+		case Types.REAL:
+			return "java.lang.Float"; //$NON-NLS-1$
+	
+		case Types.FLOAT:
+		case Types.DOUBLE:
+			return "java.lang.Double"; //$NON-NLS-1$
+	
+		case Types.CHAR:
+		case Types.VARCHAR:
+		case Types.LONGVARCHAR:
+			if (!isOpaqueBinary) {
+				return "java.lang.String"; //$NON-NLS-1$
+			}
+	
+			return "[B";
+	
+		case Types.BINARY:
+		case Types.VARBINARY:
+		case Types.LONGVARBINARY:
+	
+			if (mysqlTypeIfKnown == MysqlDefs.FIELD_TYPE_GEOMETRY) {
+				return "[B";
+			} else if (isBinaryOrBlob) {
+				return "[B";
+			} else {
+				return "java.lang.String";
+			}
+	
+		case Types.DATE:
+			return "java.sql.Date"; //$NON-NLS-1$
+	
+		case Types.TIME:
+			return "java.sql.Time"; //$NON-NLS-1$
+	
+		case Types.TIMESTAMP:
+			return "java.sql.Timestamp"; //$NON-NLS-1$
+	
+		default:
+			return "java.lang.Object"; //$NON-NLS-1$
+		}
+	}
+
+	// --------------------------JDBC 2.0-----------------------------------
+	
+	/**
+	 * JDBC 2.0
+	 * 
+	 * <p>
+	 * Return the fully qualified name of the Java class whose instances are
+	 * manufactured if ResultSet.getObject() is called to retrieve a value from
+	 * the column. ResultSet.getObject() may return a subClass of the class
+	 * returned by this method.
+	 * </p>
+	 * 
+	 * @param column
+	 *            the column number to retrieve information for
+	 * 
+	 * @return the fully qualified name of the Java class whose instances are
+	 *         manufactured if ResultSet.getObject() is called to retrieve a
+	 *         value from the column.
+	 * 
+	 * @throws SQLException
+	 *             if an error occurs
+	 */
+	public String getColumnClassName(int column) throws SQLException {
+		Field f = getField(column);
+	
+		return getClassNameForJavaType(f.getSQLType(), 
+				f.isUnsigned(), 
+				f.getMysqlType(), 
+				f.isBinary() || f.isBlob(),
+				f.isOpaqueBinary()); 
+	}
 }
