@@ -414,7 +414,8 @@ public class ServerPreparedStatement extends PreparedStatement {
 	/**
 	 * @see java.sql.PreparedStatement#clearParameters()
 	 */
-	public void clearParameters() throws SQLException {
+	public synchronized void clearParameters() throws SQLException {
+		checkClosed();
 		clearParametersInternal(true);
 	}
 
@@ -666,7 +667,8 @@ public class ServerPreparedStatement extends PreparedStatement {
 	 */
 	protected com.mysql.jdbc.ResultSet executeInternal(int maxRowsToRetrieve,
 			Buffer sendPacket, boolean createStreamingResultSet,
-			boolean queryIsSelectOnly, boolean unpackFields)
+			boolean queryIsSelectOnly, boolean unpackFields, 
+			boolean isBatch)
 			throws SQLException {
 		this.numberOfExecutions++;
 
@@ -855,58 +857,61 @@ public class ServerPreparedStatement extends PreparedStatement {
 		}
 		
 		
-		if (this.connection != null && 
-				this.connection.getAutoGenerateTestcaseScript()) {
-			dumpCloseForTestcase();
-		}
-		
-		synchronized (this.connection.getMutex()) {
+		if (this.connection != null) {
 			
-			
-			//
-			// Don't communicate with the server if we're being
-			// called from the finalizer...
-			// 
-			// This will leak server resources, but if we don't do this,
-			// we'll deadlock (potentially, because there's no guarantee
-			// when, what order, and what concurrency finalizers will be
-			// called with). Well-behaved programs won't rely on finalizers
-			// to clean up their statements.
-			//
-			
-			SQLException exceptionDuringClose = null;
-			
-			
-			if (calledExplicitly) {
-				try {
-					
-					MysqlIO mysql = this.connection.getIO();
-					
-					Buffer packet = mysql.getSharedSendPacket();
-					
-					packet.writeByte((byte) MysqlDefs.COM_CLOSE_STATEMENT);
-					packet.writeLong(this.serverStatementId);
-					
-					mysql.sendCommand(MysqlDefs.COM_CLOSE_STATEMENT, null,
-							packet, true, null);
-				} catch (SQLException sqlEx) {
-					exceptionDuringClose = sqlEx;
-				}
-				
+			if (this.connection.getAutoGenerateTestcaseScript()) {
+				dumpCloseForTestcase();
 			}
-			
-			
-			super.realClose(calledExplicitly);
-			
-			
-			clearParametersInternal(false);
-			this.parameterBindings = null;
-			
-			this.parameterFields = null;
-			this.resultFields = null;
-			
-			if (exceptionDuringClose != null) {
-				throw exceptionDuringClose;
+		
+			synchronized (this.connection) {
+				synchronized (this.connection.getMutex()) {
+									
+					//
+					// Don't communicate with the server if we're being
+					// called from the finalizer...
+					// 
+					// This will leak server resources, but if we don't do this,
+					// we'll deadlock (potentially, because there's no guarantee
+					// when, what order, and what concurrency finalizers will be
+					// called with). Well-behaved programs won't rely on finalizers
+					// to clean up their statements.
+					//
+					
+					SQLException exceptionDuringClose = null;
+					
+					
+					if (calledExplicitly) {
+						try {
+							
+							MysqlIO mysql = this.connection.getIO();
+							
+							Buffer packet = mysql.getSharedSendPacket();
+							
+							packet.writeByte((byte) MysqlDefs.COM_CLOSE_STATEMENT);
+							packet.writeLong(this.serverStatementId);
+							
+							mysql.sendCommand(MysqlDefs.COM_CLOSE_STATEMENT, null,
+									packet, true, null);
+						} catch (SQLException sqlEx) {
+							exceptionDuringClose = sqlEx;
+						}
+						
+					}
+					
+					
+					super.realClose(calledExplicitly);
+					
+					
+					clearParametersInternal(false);
+					this.parameterBindings = null;
+					
+					this.parameterFields = null;
+					this.resultFields = null;
+					
+					if (exceptionDuringClose != null) {
+						throw exceptionDuringClose;
+					}
+				}
 			}
 		}
 	}
@@ -1132,7 +1137,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 						this.connection.getId(), this.statementId, -1, System
 								.currentTimeMillis(), (int) (System
 								.currentTimeMillis() - begin), null,
-						new Throwable(), null));
+						new Throwable(), truncateQueryToLog(asSql(true))));
 			}
 
 			com.mysql.jdbc.ResultSet rs = mysql.readAllResults(this,
@@ -1155,7 +1160,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 				long elapsedTime = System.currentTimeMillis() - begin;
 
 				if (this.connection.getLogSlowQueries()
-						&& (elapsedTime > this.connection
+						&& (elapsedTime >= this.connection
 								.getSlowQueryThresholdMillis())) {
 					StringBuffer mesgBuf = new StringBuffer(
 							48 + this.originalSql.length());
@@ -1163,9 +1168,16 @@ public class ServerPreparedStatement extends PreparedStatement {
 							.getString("ServerPreparedStatement.15")); //$NON-NLS-1$
 					mesgBuf.append(this.connection
 							.getSlowQueryThresholdMillis());
+					mesgBuf.append(Messages
+							.getString("ServerPreparedStatement.15a")); //$NON-NLS-1$
+					mesgBuf.append(elapsedTime);
 					mesgBuf.append(Messages
 							.getString("ServerPreparedStatement.16")); //$NON-NLS-1$
+					
+					mesgBuf.append("as prepared: ");
 					mesgBuf.append(this.originalSql);
+					mesgBuf.append("\n\n with parameters bound:\n\n");
+					mesgBuf.append(asSql(true));
 
 					this.connection.getLog().logWarn(mesgBuf.toString());
 
@@ -1182,6 +1194,10 @@ public class ServerPreparedStatement extends PreparedStatement {
 				}
 			}
 			
+			if (mysql.hadWarnings()) {
+	            mysql.scanForAndThrowDataTruncation();
+	        }
+			
 			return rs;
 		}
 	}
@@ -1314,7 +1330,7 @@ public class ServerPreparedStatement extends PreparedStatement {
 							this.connection.getId(), this.statementId, -1,
 							System.currentTimeMillis(), (int) (System
 									.currentTimeMillis() - begin), null,
-							new Throwable(), sql));
+							new Throwable(), truncateQueryToLog(sql)));
 				}
 
 				if (this.parameterCount > 0) {
@@ -1372,6 +1388,23 @@ public class ServerPreparedStatement extends PreparedStatement {
 		}
 	}
 
+	private String truncateQueryToLog(String sql) {
+		String query = null;
+		
+		if (sql.length() > this.connection.getMaxQuerySizeToLog()) {
+			StringBuffer queryBuf = new StringBuffer(
+					this.connection.getMaxQuerySizeToLog() + 12);
+			queryBuf.append(sql.substring(0, this.connection.getMaxQuerySizeToLog()));
+			queryBuf.append(Messages.getString("MysqlIO.25"));
+			
+			query = queryBuf.toString();
+		} else {
+			query = sql;
+		}
+		
+		return query;
+	}
+
 	private void serverResetStatement() throws SQLException {
 		synchronized (this.connection.getMutex()) {
 
@@ -2185,11 +2218,13 @@ public class ServerPreparedStatement extends PreparedStatement {
 					.getString("ServerPreparedStatement.24") //$NON-NLS-1$
 					+ ioEx.toString(), SQLError.SQL_STATE_GENERAL_ERROR);
 		} finally {
-			if (inStream != null) {
-				try {
-					inStream.close();
-				} catch (IOException ioEx) {
-					; // ignore
+			if (this.connection.getAutoClosePStmtStreams()) {
+				if (inStream != null) {
+					try {
+						inStream.close();
+					} catch (IOException ioEx) {
+						; // ignore
+					}
 				}
 			}
 		}
@@ -2250,11 +2285,13 @@ public class ServerPreparedStatement extends PreparedStatement {
 					.getString("ServerPreparedStatement.25") //$NON-NLS-1$
 					+ ioEx.toString(), SQLError.SQL_STATE_GENERAL_ERROR);
 		} finally {
-			if (inStream != null) {
-				try {
-					inStream.close();
-				} catch (IOException ioEx) {
-					; // ignore
+			if (this.connection.getAutoClosePStmtStreams()) {
+				if (inStream != null) {
+					try {
+						inStream.close();
+					} catch (IOException ioEx) {
+						; // ignore
+					}
 				}
 			}
 		}
