@@ -27,13 +27,9 @@ package testsuite.regression;
 import java.io.ByteArrayInputStream;
 import java.io.File;
 import java.io.FileInputStream;
-import java.io.BufferedOutputStream;
-import java.io.FileOutputStream;
-import java.io.IOException;
 import java.io.StringReader;
 import java.sql.Blob;
 import java.sql.Connection;
-import java.sql.DriverManager;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.Statement;
@@ -75,6 +71,11 @@ public class BlobRegressionTest extends BaseTestCase {
 	 *             ...
 	 */
 	public void testBug2670() throws Exception {
+		
+		if (isRunningOnJdk131()) {
+			return; // test not valid on JDK-1.3.1
+		}
+		
 		try {
 			byte[] blobData = new byte[32];
 
@@ -253,6 +254,11 @@ public class BlobRegressionTest extends BaseTestCase {
 	 *             if the test fails.
 	 */
 	public void testBug8096() throws Exception {
+		
+		if (isRunningOnJdk131()) {
+			return; // test not valid on JDK-1.3.1
+		}
+		
 		int dataSize = 256;
 
 		Properties props = new Properties();
@@ -287,7 +293,7 @@ public class BlobRegressionTest extends BaseTestCase {
 			this.rs = ps.executeQuery();
 
 			if (this.rs.next()) {
-				Blob b = rs.getBlob("BLOB_DATA");
+				Blob b = this.rs.getBlob("BLOB_DATA");
 				b.setBytes(1, testData);
 			}
 
