@@ -44,7 +44,7 @@ import com.mysql.jdbc.SQLError;
  * DOCUMENT ME!
  * 
  * @author Mark Matthews
- * @version $Id: StatementsTest.java 4050 2005-08-11 19:15:17Z mmatthews $
+ * @version $Id: StatementsTest.java 5034 2006-03-09 20:56:13Z mmatthews $
  */
 public class StatementsTest extends BaseTestCase {
 	private static final int MAX_COLUMN_LENGTH = 255;
@@ -913,6 +913,108 @@ public class StatementsTest extends BaseTestCase {
 		}
 	}
 
+	public void testStatementRewriteBatch() throws SQLException {
+		Properties props = new Properties();
+		props.setProperty("rewriteBatchedStatements", "true");
+		Connection multiConn = getConnectionWithProps(props);
+		createTable("testStatementRewriteBatch", "(pk_field INT PRIMARY KEY NOT NULL AUTO_INCREMENT, field1 INT)");
+		Statement multiStmt = multiConn.createStatement();
+		multiStmt.addBatch("INSERT INTO testStatementRewriteBatch(field1) VALUES (1)");
+		multiStmt.addBatch("INSERT INTO testStatementRewriteBatch(field1) VALUES (2)");
+		multiStmt.addBatch("INSERT INTO testStatementRewriteBatch(field1) VALUES (3)");
+		multiStmt.addBatch("INSERT INTO testStatementRewriteBatch(field1) VALUES (4)");
+		multiStmt.addBatch("UPDATE testStatementRewriteBatch SET field1=5 WHERE field1=1");
+		multiStmt.addBatch("UPDATE testStatementRewriteBatch SET field1=6 WHERE field1=2 OR field1=3");
+		
+		int[] counts = multiStmt.executeBatch();
+		ResultSet genKeys = multiStmt.getGeneratedKeys();
+		
+		for (int i = 1; i < 5; i++) {
+			genKeys.next();
+			assertEquals(i, genKeys.getInt(1));
+		}
+		
+		assertEquals(counts.length, 6);
+		assertEquals(counts[0], 1);
+		assertEquals(counts[1], 1);
+		assertEquals(counts[2], 1);
+		assertEquals(counts[3], 1);
+		assertEquals(counts[4], 1);
+		assertEquals(counts[5], 2);
+		
+		this.rs = multiStmt.executeQuery("SELECT field1 FROM testStatementRewriteBatch ORDER BY field1");
+		assertTrue(this.rs.next());
+		assertEquals(this.rs.getInt(1), 4);
+		assertTrue(this.rs.next());
+		assertEquals(this.rs.getInt(1), 5);
+		assertTrue(this.rs.next());
+		assertEquals(this.rs.getInt(1), 6);
+		assertTrue(this.rs.next());
+		assertEquals(this.rs.getInt(1), 6);
+
+		createTable("testStatementRewriteBatch", "(pk_field INT PRIMARY KEY NOT NULL AUTO_INCREMENT, field1 INT)");
+		props.clear();
+		props.setProperty("rewriteBatchedStatements", "true");
+		props.setProperty("sessionVariables", "max_allowed_packet=1024");
+		multiConn = getConnectionWithProps(props);
+		multiStmt = multiConn.createStatement();
+		
+		for (int i = 0; i < 1000; i++) {
+			multiStmt.addBatch("INSERT INTO testStatementRewriteBatch(field1) VALUES (" + i + ")");
+		}
+		
+		multiStmt.executeBatch();
+		genKeys = multiStmt.getGeneratedKeys();
+		
+		for (int i = 1; i < 1000; i++) {
+			genKeys.next();
+			assertEquals(i, genKeys.getInt(1));
+		}
+		
+		createTable("testStatementRewriteBatch", "(pk_field INT PRIMARY KEY NOT NULL AUTO_INCREMENT, field1 INT)");
+		
+		props.clear();
+		props.setProperty("useServerPrepStmts", "false");
+		props.setProperty("rewriteBatchedStatements", "true");
+		multiConn = getConnectionWithProps(props);
+		PreparedStatement pStmt = multiConn.prepareStatement("INSERT INTO testStatementRewriteBatch(field1) VALUES (?)", 
+				Statement.RETURN_GENERATED_KEYS);
+		
+		for (int i = 0; i < 1000; i++) {
+			pStmt.setInt(1, i);
+			pStmt.addBatch();
+		}
+		
+		pStmt.executeBatch();
+		genKeys = pStmt.getGeneratedKeys();
+		
+		for (int i = 1; i < 1000; i++) {
+			genKeys.next();
+			assertEquals(i, genKeys.getInt(1));
+		}
+		
+		createTable("testStatementRewriteBatch", "(pk_field INT PRIMARY KEY NOT NULL AUTO_INCREMENT, field1 INT)");
+		props.setProperty("useServerPrepStmts", "false");
+		props.setProperty("rewriteBatchedStatements", "true");
+		props.setProperty("sessionVariables", "max_allowed_packet=1024");
+		multiConn = getConnectionWithProps(props);
+		pStmt = multiConn.prepareStatement("INSERT INTO testStatementRewriteBatch(field1) VALUES (?)", 
+				Statement.RETURN_GENERATED_KEYS);
+		
+		for (int i = 0; i < 1000; i++) {
+			pStmt.setInt(1, i);
+			pStmt.addBatch();
+		}
+		
+		pStmt.executeBatch();
+		genKeys = pStmt.getGeneratedKeys();
+		
+		for (int i = 1; i < 1000; i++) {
+			genKeys.next();
+			assertEquals(i, genKeys.getInt(1));
+		}
+	}
+	
 	/**
 	 * DOCUMENT ME!
 	 * 
